/*********************	Written by Christian Nordahl  ***********************/
/*	
 *	This file tests gaussian generators based in C, and sends the results back in a struct
 */

#define K 4					/* constraint length */
#define NUMSTATES 8			/* 2^(K - 1) -- change as required */

#define PI 3.141592654		/* circumference of circle divided by diameter */

#define N 212				/* Block sizes (N/4 = bytes) */
#define R 3					/* code rate number: R=1, code rate r=1/3; R=2, code rate r=2/5;     */
/* R=3, code rate r=1/2; R=4, code rate r=2/3; R=5, code rate r=3/4; */
/* R=6, code rate r=4/5; R=7, code rate r=6/7; */

#define DECITER 8			/* the number of decoding iterations */
#define MINFRAMES 1000		/* the minimum number of frames to be simulated */

#define LOEBN0 1.5			/* minimum Eb/No at which to test */
#define HIEBN0 3.001		/* maximum Eb/No at which to test */
#define EBN0STEP 0.5		/* Eb/No increment for test driver */

typedef struct { float *errors, *Lcer; int errorsLength, frames, snrSteps;} Results;

#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <time.h>

void gen01datDUMMY(int data_len);
void permutation(int *alpha);
void makenoise(float eb_ovr_n0, int encd_block_len, float rate, int *in_array, float *out_array);
void gngauss(float mean, float sigma, int encd_block_len, float *gauss1, float *gauss2);

//int main() {
//	printf("hello world\n");
//	int fjas = 0;
//	return fjas;
//}

Results *testGauss()
{
	int i,j,ll,iter,t,start,frame;  /* loop variables */
	int encd_block_len,total_len;   /* the length of the encoded block depending on the code rate */
	int M,msg_length,channel_len;   /* parameter of encoded block length */
	int dec_iter,snrIter;                 /* turbo decoding iteration */
	
	float *noise,*Lcene;                  /* noisy data array */
	float eb_ovr_n0,rate,L_c;
	
	/********************************************************************************************************************/
	/*
	 * The following is included so that we have the same number of rand() calls both in the original dvb-rcs file 
	 * and this testing file.  */
	
#if R == 1
	rate = 1.0/3.0;
	M = N;
	encd_block_len = 2*N + M;
#endif
	
#if R == 2
	rate = 2.0/5.0;
	M = N/2;
	encd_block_len = 2*N + M;
#endif
	
#if R == 3
	rate = 1.0/2.0;
	M = N;
	encd_block_len = N + M;
#endif
	
#if R == 4
	rate = 2.0/3.0;
	M = N/2;
	encd_block_len = N + M;
#endif
	
#if R == 5
	rate = 3.0/4.0;
	if (N%3 == 0) M = N/3;
	else if (N%3 == 1) M = (N-4)/3 + 2;
	else if (N%3 == 2) M = (N-8)/3 + 3;
	encd_block_len = N + M;
#endif
	
#if R == 6
	rate = 4.0/5.0;
	M = N/4;
	encd_block_len = N + M;
#endif
	
#if R == 7
	rate = 6.0/7.0;
	if (N%3 == 0) M = N/6;
	else if (N%3 == 1) M = (N-4)/6 + 1;
	else if (N%3 == 2) M = (N-8)/6 + 2;
	encd_block_len = N + M;
#endif

	int snrIterations = floor( (HIEBN0 - LOEBN0) / EBN0STEP ) + 1;
	
	msg_length = 2*N;
	channel_len = 2*encd_block_len;
	total_len = channel_len*MINFRAMES;
	
	/********************************************************************************************************************/	
	
	noise = malloc( total_len * snrIterations * sizeof(float) );
	if (noise == NULL) {
		printf("\n gausstest.c:  error allocating noise array, aborting!\n");
		exit(1);
	}
    
    Lcene = malloc( snrIterations * sizeof(float) );
	if (Lcene == NULL) {
		printf("\n gausstest.c:  error allocating Lc array, aborting!\n");
		exit(1);
	}
	
	int iterationshift,ebShift;
	float mean, eb, sn_ratio, sigma, signal_1, signal_2;
	snrIter = 0;
	
	for (eb_ovr_n0 = LOEBN0; eb_ovr_n0 <= HIEBN0; eb_ovr_n0 += EBN0STEP) {
		
		srand(time(NULL));					/* re-seeding the random number generator */
		L_c = 4 * rate * pow(10,eb_ovr_n0/10);
		ebShift = snrIter*2*encd_block_len*MINFRAMES;

        sn_ratio = (float) pow(10, ( eb_ovr_n0 / 10) );
        sigma =  (float) sqrt (eb / (2 * sn_ratio * rate) );
		
        *(Lcene + snrIter) = sigma;

		for (iter = 0; iter < MINFRAMES; iter++) {
			
			gen01datDUMMY(msg_length);
			
			float *gauss1;
			float *gauss2;
			
			mean = 0;
			eb = 1;
			
			gauss1 = malloc( encd_block_len * sizeof(float) );
			if (gauss1 == NULL) {
				printf("\n addnoise:  error allocating cos I noise array, aborting!\n");
				exit(1);
			}
			
			gauss2 = malloc( encd_block_len * sizeof(float) );
			if (gauss2 == NULL) {
				printf("\n addnoise:  error allocating sin Q noise array, aborting!\n");
				exit(1);
			}
			
			gngauss(mean, sigma, encd_block_len, gauss1, gauss2);
			
			int iterationshift = iter * 2 * encd_block_len;
			
			for (t = 0; t < encd_block_len; t++) {
				*( noise + 2*t + iterationshift + ebShift) = *(gauss1+t);
				*( noise + 2*t + 1 + iterationshift + ebShift) = *(gauss2+t);
			}
			
			free(gauss1);
			free(gauss2);
			
		} /* end of for loop */
            
        sn_ratio = 0;
        sigma = 0;
		
		snrIter += 1;
	}
	
	Results *res;
	res = malloc( sizeof(Results) );
	if (res == NULL) {
		printf("\n turbo_dvbrcs.c: error allocating result struct, aborting!");
		exit(1);
	}
	res->errors = noise;
	res->Lcer = Lcene;
	res->errorsLength = total_len;
	res->frames = MINFRAMES;
	res->snrSteps = snrIterations;
	return res;
}

/*******************/
/* 0/1 Generation  */
/*******************/
void gen01datDUMMY(int data_len)
{	
	rand();
	int tt,dummy;							/* time */
	for (tt = 0; tt < data_len; tt++) {	/* generate random data, write to output array */
		dummy = (int)( rand() / (RAND_MAX / 2) > 0.5 );
	}
}


void gngauss(float mean, float sigma, int encd_block_len, float *gauss1, float *gauss2) 
{ 
	int i;
	double u, r;                         /* uniform and Rayleigh random variables */
	
	for (i=0; i<encd_block_len; i++){
		u = (double)rand() / RAND_MAX;     /* uniformly distributed random number u between 0 and 1 - 1E-9*/
		if (u == 1.0) u = 0.999999999;
		
		r = sigma * sqrt( 2.0 * log( 1.0 / (1.0 - u) ) ); /* generate a Rayleigh-distributed random number r using u */
		
		u = (double)rand() / RAND_MAX;             /* generate another uniformly-distributed random number u as before*/
		if (u == 1.0) u = 0.999999999;
		
		*(gauss1+i) = mean + r * cos(2 * PI * u);  /* generate two Gaussian-distributed random number using r and u */
		*(gauss2+i) = mean + r * sin(2 * PI * u);
	} 
}