/* ********************	Written by Christian Nordahl  ***********************
 *
 * My gateway routine for Matlab <--> C crossover. Takes no input from Matlab,
 * but returns the results so they can be saved and manipulated in the more
 * user friendly Matlab environment.
 */

#include "mex.h"
#include "matrix.h"
#include "turbo_dvbrcs.h"

#define ERRRORPOS plhs[0]
#define FRAMES plhs[1]
#define ONEC plhs[2]
#define ZEROC plhs[3]
#define BERS plhs[4]
#define FERS plhs[5]
#define BLOCKSIZE plhs[6]
#define RATENR plhs[7]
#define SNRITERS plhs[8]


Results *dvbrcsSimulation();

void mexFunction(
				 int nlhs, mxArray *plhs[], 
				 int nrhs, const mxArray *prhs[]) 
{
	int intIters, intBlocks, intRate, iterLoop, errLoop, msgLength;
	int *errorsTemp, *framesTemp, *onesTemp, *zerosTemp;
	double *berTemp, *ferTemp;
	double *errorsOut, *framesOut, *onesOut, *zerosOut, *berOut, *ferOut, *blocksOut, *rateOut, *itersOut;
	
    Results *result;
    
    /* Check for proper number of arguments */
    if (nrhs != 0) { 
        mexErrMsgTxt("No input required."); 
    }
    
    if (nlhs != 9) {
        mexErrMsgTxt("Needs nine output arguments."); 
    }
    
    result = dvbrcsSimulation();
	intIters = result->snrIters;
	intBlocks = result->blockLength;
	msgLength = 2*intBlocks;

	ERRRORPOS  = mxCreateDoubleMatrix(intIters, msgLength, mxREAL);
	errorsOut = mxGetPr(ERRRORPOS);
	errorsTemp = result->errorPos;

	for(iterLoop=0; iterLoop<intIters; iterLoop++) {
		for(errLoop=0; errLoop<msgLength; errLoop++) {
			*(errorsOut + errLoop + iterLoop*msgLength) = (double) ( *(errorsTemp + errLoop + iterLoop*msgLength));
		}
	}
	
	FRAMES = mxCreateDoubleMatrix(intIters, 1, mxREAL);
	framesOut = mxGetPr(FRAMES);
	framesTemp = result->frames;
	
	ONEC = mxCreateDoubleMatrix(intIters, 1, mxREAL);
	onesOut = mxGetPr(ONEC);
	onesTemp = result->onecheck;
	
	ZEROC = mxCreateDoubleMatrix(intIters, 1, mxREAL);
	zerosOut = mxGetPr(ZEROC);
	zerosTemp = result->zerocheck;
	
	BERS = mxCreateDoubleMatrix(intIters, 1, mxREAL);
	berOut = mxGetPr(BERS);
	berTemp = result->ber;
	
	FERS = mxCreateDoubleMatrix(intIters, 1, mxREAL);
	ferOut = mxGetPr(FERS);
	ferTemp = result->fer;
	
	for(iterLoop=0; iterLoop<intIters; iterLoop++) {
		*(framesOut + iterLoop) = (double) ( *(framesTemp + iterLoop));
		*(onesOut + iterLoop) = (double) ( *(onesTemp + iterLoop));
		*(zerosOut + iterLoop) = (double) ( *(zerosTemp + iterLoop));
		
		*(berOut + iterLoop) = *(berTemp + iterLoop);
		*(ferOut + iterLoop) = *(ferTemp + iterLoop);
	}
	
	
	BLOCKSIZE = mxCreateDoubleMatrix(1, 1, mxREAL);
	blocksOut = mxGetPr(BLOCKSIZE);
	*blocksOut = (double) intBlocks;
	
	RATENR = mxCreateDoubleMatrix(1, 1, mxREAL);
	rateOut = mxGetPr(RATENR);
	intRate = result->rate;
	*rateOut = (double) intRate;
	
	SNRITERS = mxCreateDoubleMatrix(1, 1, mxREAL);
	itersOut = mxGetPr(SNRITERS);
	*itersOut = (double) intIters;
	
	free(errorsTemp);
	free(framesTemp);
	free(onesTemp);
	free(zerosTemp);
	free(berTemp);
	free(ferTemp);
	
	free(result);
}
