function [circStates,modulus] = findCircularStates(blocksizeN)
    
    % Can't invert G if its raised to the power of the generator period
    if(mod(blocksizeN,15)==0)
        'The block size cannot be a multiple of the generator period'
        circStates = -1;
        modulus = -1;
        return
    end
    
    if(mod(blocksizeN,15)==10)
        'Seems to experience some problems with numerical precision with a modulus of ten. Instead done step by step in the command window'
        circStates = -1;
        modulus = -1;
        return
    end
        
    % Generator matrix with feedback from state 3 and 4 %
    G =[0 0 1 1;
        1 0 0 0;
        0 1 0 0;
        0 0 1 0;];
    
    % Array for converting states to decimal values
    twoPowers = [8 4 2 1]';
    
    % Possible state-values
    allStates = [0 0 0 0;
              0 0 0 1;
              0 0 1 0;
              0 0 1 1;
              0 1 0 0;
              0 1 0 1;
              0 1 1 0;
              0 1 1 1;
              1 0 0 0;
              1 0 0 1;
              1 0 1 0;
              1 0 1 1;
              1 1 0 0;
              1 1 0 1;
              1 1 1 0;
              1 1 1 1;];
    
    % Need to piece up the raising of G to power to prevent inaccuracy
    % due to overlarge numbers
    looping = floor(blocksizeN/50)
    rest = blocksizeN - (looping*50)
    temp = mod(G^rest,2);
    
    for i=1:looping
        temp = mod((temp*G^50),2);
    end
    
    temp = mod((temp + eye(4)),2);
    
    % Calculating the circular states for the given block size
    temp2 = (mod((temp\allStates'),2))';
    
    circStates = temp2*twoPowers;
    modulus = mod(blocksizeN,15);
    return
end