/*
 *  turboPhiTrellis.c
 *  DvbrcsTurboCodes
 *
 *  written by Christian Nordahl
 *
 */



#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>


#define K 5					/* constraint length */
#define SYMBOLS 4			/* number of possible symbols */
#define NUMSTATES 16		/* 2^(K - 1) -- change as required */

#define PI 3.141592654		/* circumference of circle divided by diameter */

#define N 752				/* Block sizes (N/4 = bytes) */
#define R 1					/* code rate number: R=1, code rate r=1/3; R=2, code rate r=2/5;     */
							/* R=3, code rate r=1/2; R=4, code rate r=2/3; R=5, code rate r=3/4; */
							/* R=6, code rate r=4/5; R=7, code rate r=6/7; */

#define DECITER 8			/* the number of decoding iterations */

#define LOEBN0 1.05			/* minimum Eb/No at which to test */
#define HIEBN0 1.20			/* maximum Eb/No at which to test */
#define EBN0STEP 0.1		/* Eb/No increment for test driver */


void main(){
	//int g[4][K] = { 
//					{1,0, 0,1,1},       /* Recursion: 23 octally */
//					{1,1, 1,0,1},       /* Y-redundance: 35 octally */
//					{1,0, 1,0,1},		/* W-redundance: 21 octally */
//					{0,1, 0,1,0}};      /* Helper variable for updating the shift register : 12 octally */

	printf("main som ikke gjør no, Xcode krever en main-fil for å builde prosjektet");

}


void tphi_redundancyEnc(int g[4][K], int *inputA_array, int *inputB_array, int *Y_array, int *W_array, int state0[K-1], int stateN[K-1])
{
	int j, t;					/* loop variable, bit time */
	int shift_register[K];      /* the RSC encoder shift register, with the current input at index '0' */
	int y,w;					/* output redundancy bits */
	
	shift_register[0] = 0;		/* Initialize the shift register */
	for (j=0; j<K-1; j++){
		shift_register[j+1] = state0[j];
	}
	
	for (t=0; t<N; t++){		/* The encoding of redundance bits */
		shift_register[0] = *(inputA_array+t) ^ *(inputB_array+t);
		y = 0;
		w = 0;
		
		for (j=1; j<K; j++){   /* Compute the feedback */
			shift_register[0] ^= (shift_register[j] & g[0][j]);
		}
		
		for (j=0; j<K; j++){   /* Compute the redundancies */
			y ^= (shift_register[j] & g[1][j]);
			w ^= (shift_register[j] & g[2][j]);
		}
		
		for(j=K-2; j>=0; j--){   /* Shift the register */
			shift_register[j+1] = shift_register[j] ^ (*(inputB_array+t) & g[3][j]);
		}
		
		*(Y_array+t) = y;
		*(W_array+t) = w;
	} /* end of t-loop and redundance encoding */

	for (j=0; j<K-1; j++){ /* return the final state */
		stateN[j] = shift_register[j+1];
	}
}


/*************************************************/
/* Set up the trellis for a given code generator */
/*************************************************/
void deci2bin(int d, int size, int *b);
int bin2deci(int *b, int size);
int nxt_state(int g[4][K], int current_state, int input, int *memory_contents);

void trellis(int g[4][K], int nextstate[NUMSTATES][SYMBOLS], int nextoutput[NUMSTATES][NUMSTATES],
			 int prestate[NUMSTATES][SYMBOLS], int preoutput[NUMSTATES][NUMSTATES])
{
	int i,j,l;                   /* loop variables */
	int memory_contents[K];      /* feedback bit + conv. encoder sr */
	int encd_output[2];          /* store trial encoder output */
	int next_state;
	
	for (i = 0; i < NUMSTATES; i++) {	/* Initialize the data structures */
		for (j = 0; j < NUMSTATES; j++){
			preoutput[i][j] = 0;
			nextoutput[i][j] = 0;
		}
		for (j = 0; j < SYMBOLS; j++){
			prestate[i][j] = 0;
			nextstate[i][j] = 0;
		}
	}
	
	/* Generate the matrices */
	for (j = 0; j < NUMSTATES; j++) {
		for (l = 0; l < SYMBOLS; l++) {
		
			/* Next state, given current state and input */
			next_state = nxt_state(g, j, l, memory_contents);
			
			/* Generate the output */
			encd_output[0] = 0;
			encd_output[1] = 0;
			for (i = 0; i < K; i++){
				encd_output[0] ^= memory_contents[i] & g[1][i];
				encd_output[1] ^= memory_contents[i] & g[2][i];
			}
			
			nextstate[j][l] = next_state;		
			nextoutput[j][next_state] = bin2deci(encd_output,2);
			
			prestate[next_state][l] = j;
			preoutput[next_state][j] = nextoutput[j][next_state];
		}  /* end of l for loop */
	}  /* end of j for loop */
}


/* ************************************************************************** */
int nxt_state(int g[4][K], int current_state, int input, int *memory_contents) {
	int binary_state[K - 1];              /* binary value of current state */
	int binary_input[2];                  /* binary value of input */
	int next_state_binary[K - 1];         /* binary value of next state */
	int next_state;                       /* decimal value of next state */
	int feedback;                         /* feedback bit */
	int i;                                /* loop variable */
	
	deci2bin(current_state, K - 1, binary_state);	/* convert the decimal value of the current state number to binary */
	deci2bin(input, 2, binary_input);				/* convert the decimal value of the input to binary */
	
	/* Get the feedback bit */
	feedback = binary_input[0] ^ binary_input[1];
	for (i=0; i<K-1; i++){
		feedback ^= binary_state[i] & g[0][i+1];	
	}
	
	/* Given the input and current state number, compute the next state number */
	next_state_binary[0] = feedback;
	for (i = 1; i < K - 1; i++){
		next_state_binary[i] = binary_state[i - 1] ^ (binary_input[1]  & g[3][i]);
	}
		
	/* Convert state number to decimal */
	next_state = bin2deci(next_state_binary, K - 1);
	
	/* Load the memory contents with the current state, for use by the trellis method*/
	memory_contents[0] = feedback;
	for (i = 0; i < K-1; i++){
		memory_contents[i+1] = binary_state[i];
	}
	
	return(next_state);
}

void deci2bin(int d, int size, int *b) {  /* converts a decimal number to a binary number */
	int i;
	
	for(i = 0; i < size; i++)
		b[i] = 0;
	
	b[size - 1] = d & 0x01;
	
	for (i = size - 2; i >= 0; i--) {
		d = d >> 1;
		b[i] = d & 0x01;
	}
}

int bin2deci(int *b, int size) {         /* converts a binary number to decimal number */
	int i, d;
	
	d = 0;
	
	for (i = 0; i < size; i++)
		d = d + (int)pow(2, size - i - 1) * b[i];
	
	return(d);
}